/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.MainWindow : Gtk.Window {
    AdminPage admin_page;
    OtherPage other_page;
    PayPage pay_page;
    BalancePage balance_page;
    ProductsPage products_page;
    RefillPage refill_page;
    CardPage card_page;
    StockPage stock_page;

    Gtk.Grid header_bar;
    Gtk.Label title_label;
    Gtk.Stack stack;

    Gtk.Label notification_label;
    Gtk.Revealer app_notification;

    Gtk.Stack main_stack;
    Reader reader;
    unowned Database database;
    public MainWindow () {
        
    }

    construct {
        Gtk.Settings.get_default ().gtk_application_prefer_dark_theme = true;
        database = Database.get_default ();

        header_bar = new Gtk.Grid ();
        header_bar.hexpand = true;
        header_bar.get_style_context ().add_class ("header-bar");
        header_bar.get_style_context ().add_class (Gtk.STYLE_CLASS_TITLEBAR);

        title_label = new Gtk.Label ("");
        title_label.hexpand = true;
        title_label.xalign = 0.5f;
        title_label.margin = 12;
        header_bar.attach (title_label, 1, 0, 1, 1);

        admin_page = new AdminPage ();
        other_page = new OtherPage ();
        balance_page = new BalancePage ();
        products_page = new ProductsPage ();
        card_page = new CardPage ();
        pay_page = new PayPage (products_page.cart_view);
        refill_page = new RefillPage ();
        stock_page = new StockPage ();

        reader = new Reader ();
        reader.error_message.connect ((msg) => {
            show_message (msg);
        });

        reader.tag_found.connect ((card_id) => tag_found.begin (card_id));

        other_page.show_balance.connect (() => {
            reader.start_poll ();
            set_visible_page (balance_page);
        });
        other_page.show_refill.connect (() => {
            set_visible_page (refill_page);
        });
        other_page.show_home.connect (() => {
            set_visible_page (products_page);
        });
        other_page.show_card.connect (() => {
            card_page.check_first_show ();
            set_visible_page (card_page);
        });
        other_page.show_stock.connect (() => {
            stock_page.show_products ();
            set_visible_page (stock_page);
        });

        pay_page.show_cart.connect (() => {
            reader.stop_poll ();
            set_visible_page (products_page);
        });
        pay_page.show_home.connect (() => {
            reader.stop_poll ();
            products_page.clear ();
            set_visible_page (products_page);
        });
        pay_page.show_message.connect ((msg) => {
            show_message (msg);
        });

        balance_page.show_others.connect (() => {
            reader.stop_poll ();
            set_visible_page (other_page);
        });

        products_page.show_others.connect (() => {
            set_visible_page (other_page);
        });
        products_page.show_pay.connect (() => {
            pay_page.update_amount ();
            reader.start_poll ();
            set_visible_page (pay_page);
        });

        refill_page.show_home.connect (() => {
            set_visible_page (products_page);
        });
        refill_page.waiting_target.connect (() => {
            reader.start_poll ();
        });

        card_page.show_others.connect (() => {
            reader.stop_poll ();
            set_visible_page (other_page);
        });
        card_page.waiting_target.connect (() => {
            reader.start_poll ();
        });
        card_page.stop_waiting_target.connect (() => {
            reader.stop_poll ();
        });
        card_page.show_message.connect ((msg) => {
            show_message (msg);
        });

        stock_page.show_others.connect (() => {
            set_visible_page (other_page);
        });

        stack = new Gtk.Stack ();
        stack.add (admin_page);
        stack.add (other_page);
        stack.add (balance_page);
        stack.add (products_page);
        stack.add (pay_page);
        stack.add (refill_page);
        stack.add (card_page);
        stack.add (stock_page);

        var main_grid = new Gtk.Grid ();
        main_grid.orientation = Gtk.Orientation.VERTICAL;
        main_grid.add (header_bar);
        main_grid.add (stack);

        var no_service_grid = new Gtk.Grid ();
        no_service_grid.halign = Gtk.Align.CENTER;
        no_service_grid.valign = Gtk.Align.CENTER;
        no_service_grid.row_spacing = 6;
        no_service_grid.column_spacing = 12;
        var no_service_title = new Gtk.Label (_("Terminal Locked"));
        no_service_title.get_style_context ().add_class ("h3");
        no_service_title.halign = Gtk.Align.START;
        var no_service_description = new Gtk.Label (_("Use you Card to Unlock the Terminal"));
        no_service_description.halign = Gtk.Align.START;
        var no_service_image = new Gtk.Image.from_icon_name ("system-lock-screen", Gtk.IconSize.DIALOG);
        no_service_grid.attach (no_service_image, 0, 0, 1, 2);
        no_service_grid.attach (no_service_title, 1, 0, 1, 1);
        no_service_grid.attach (no_service_description, 1, 1, 1, 1);

        main_stack = new Gtk.Stack ();
        main_stack.transition_type = Gtk.StackTransitionType.CROSSFADE;
        main_stack.add_named (main_grid, "main");
        main_stack.add_named (no_service_grid, "no_service");

        var close_button = new Gtk.Button.from_icon_name ("close-symbolic", Gtk.IconSize.MENU);
        close_button.get_style_context ().add_class ("close-button");
        close_button.clicked.connect (() => {
            app_notification.reveal_child = false;
        });

        notification_label = new Gtk.Label (null);

        var notification_box = new Gtk.Grid ();
        notification_box.column_spacing = 12;
        notification_box.add (close_button);
        notification_box.add (notification_label);

        var notification_frame = new Gtk.Frame (null);
        notification_frame.get_style_context ().add_class ("app-notification");
        notification_frame.add (notification_box);

        app_notification = new Gtk.Revealer ();
        app_notification.margin = 3;
        app_notification.halign = Gtk.Align.CENTER;
        app_notification.valign = Gtk.Align.START;
        app_notification.add (notification_frame);

        var overlay = new Gtk.Overlay ();
        overlay.add (main_stack);
        overlay.add_overlay (app_notification);

        add (overlay);

        fullscreen ();
        show_all ();
        set_visible_page (products_page);
        stack.transition_type = Gtk.StackTransitionType.SLIDE_LEFT_RIGHT;

        database.notify["locked"].connect (lock_changed);
        lock_changed ();
    }

    public void show_message (string message) {
        notification_label.label = message;
        app_notification.reveal_child = true;
    }

    private void lock_changed () {
        if (database.locked) {
            reader.start_poll ();
            main_stack.set_visible_child_name ("no_service");
        } else {
            reader.stop_poll ();
            main_stack.set_visible_child_name ("main");
        }
    }

    private async void tag_found (string card_id) {
        if (database.locked) {
            var identified = yield database.identify_server (card_id);
            if (!identified) {
                reader.start_poll ();
            }
        } else if (stack.visible_child == pay_page) {
            if (yield pay_page.card_found (card_id)) {
                reader.stop_poll ();
            } else {
                reader.start_poll ();
            }
        } else if (stack.visible_child == balance_page) {
            reader.stop_poll ();
            yield balance_page.card_found (card_id);
        } else if (stack.visible_child == refill_page) {
            reader.stop_poll ();
            yield refill_page.card_found (card_id);
        } else if (stack.visible_child == card_page) {
            reader.stop_poll ();
            yield card_page.card_found (card_id);
        }
    }

    private void set_visible_page (Page page) {
        header_bar.get_children ().foreach ((child) => {
            child.hide ();
        });

        unowned Gtk.Widget? start_title = page.get_start_title ();
        if (start_title != null) {
            start_title.show_all ();
            if (start_title.get_parent () == null) {
                header_bar.attach (start_title, 0, 0, 1, 1);
                start_title.margin = 3;
                start_title.halign = Gtk.Align.START;
            }
        }
        unowned Gtk.Widget? end_title = page.get_end_title ();
        if (end_title != null) {
            end_title.show_all ();
            if (end_title.get_parent () == null) {
                header_bar.attach (end_title, 2, 0, 1, 1);
                end_title.margin = 3;
                end_title.halign = Gtk.Align.END;
            }
        }

        title_label.label = page.get_name ();
        title_label.visible = true;
        stack.set_visible_child (page);
    }
}
