/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.Product : GLib.Object {
    public signal void children_changed (Product product, bool added);
    public enum Category {
        DRINK,
        FOOD;
        public static Category from_string (string cat) {
            switch (cat) {
                case "drink":
                    return DRINK;
                default:
                    return FOOD;
            }
        }
    }

    public int SKU { get; set; }
    public string name { get; set; }
    private int _quantity = -1;
    public int quantity {
        get {
            if (children.is_empty) {
                return _quantity;
            } else {
                return get_child_quantity ();
            }
        }

        set {
            if (children.is_empty) {
                _quantity = value;
            }
        }
    }

    public uint price { get; set; default=uint.MAX; }
    public uint price_member { get; set; default=uint.MAX; }
    public uint price_menu { get; set; default=uint.MAX; }
    public uint price_member_menu { get; set; default=uint.MAX; }
    public bool menu_activator { get; set; default=false; }
    public Product parent { get; set; default = null; }
    public Gee.TreeSet<Product> children { get; private set; }

    public Category category { get; set; }

    public Product () {
        
    }

    private int get_child_quantity () {
        int i = 0;
        foreach (var child in children) {
            if (child.children.is_empty) {
                if (child.quantity >= 0) {
                    i += child.quantity;
                } else {
                    i += 1;
                }
            } else {
                i += child.get_child_quantity ();
            }
        }

        return i;
    }

    construct {
        children = new Gee.TreeSet<Product> ();
    }

    public static int compare (Product a, Product b) {
        return a.name.collate (b.name);
    }
}
