/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.AmountWidget : Gtk.Grid {
    public signal void accepted (int amount);

    public bool with_cents { get; set; default=false; }
    private int amount { get; set; default=0; }
    private Gtk.Label amount_label;
    private Gtk.Button add_button;

    construct {
        var amount_tag_label = new Gtk.Label (_("Amount:"));
        amount_tag_label.halign = Gtk.Align.START;
        amount_tag_label.get_style_context ().add_class ("h2");
        amount_label = new Gtk.Label (_("$0"));
        amount_label.get_style_context ().add_class ("h2");
        amount_label.halign = Gtk.Align.START;
        expand = true;
        column_homogeneous = true;
        column_spacing = 6;
        row_spacing = 6;

        var zero_button = create_number_button (0);
        var one_button = create_number_button (1);
        one_button.expand = true;
        var two_button = create_number_button (2);
        var three_button = create_number_button (3);
        var four_button = create_number_button (4);
        var five_button = create_number_button (5);
        var six_button = create_number_button (6);
        six_button.expand = true;
        var seven_button = create_number_button (7);
        var eight_button = create_number_button (8);
        var nine_button = create_number_button (9);
        attach (one_button, 0, 1, 1, 1);
        attach (two_button, 1, 1, 1, 1);
        attach (three_button, 2, 1, 1, 1);
        attach (four_button, 3, 1, 1, 1);
        attach (five_button, 4, 1, 1, 1);
        attach (six_button, 0, 2, 1, 1);
        attach (seven_button, 1, 2, 1, 1);
        attach (eight_button, 2, 2, 1, 1);
        attach (nine_button, 3, 2, 1, 1);
        attach (zero_button, 4, 2, 1, 1);

        var delete_button = new Gtk.Button.from_icon_name ("edit-clear-symbolic", Gtk.IconSize.BUTTON);
        add_button = new Gtk.Button.with_label (_("Add"));
        add_button.sensitive = false;
        add_button.get_style_context ().add_class (Gtk.STYLE_CLASS_SUGGESTED_ACTION);
        attach (delete_button, 3, 0, 1, 1);
        var amount_grid = new Gtk.Grid ();
        amount_grid.column_spacing = 12;
        amount_grid.orientation = Gtk.Orientation.HORIZONTAL;
        amount_grid.add (amount_tag_label);
        amount_grid.add (amount_label);
        attach (amount_grid, 0, 0, 3, 1);
        attach (add_button, 4, 0, 1, 1);

        add_button.clicked.connect (() => {
            accepted (amount);
            reset ();
        });

        delete_button.clicked.connect (() => {
            amount = (amount - amount % 10)/10;
            redraw_amount ();
        });

        if (with_cents) {
            amount_label.label = _("$0.00");
        }

        notify["with-cents"].connect (() => reset ());
    }

    public void reset () {
        amount = 0;
        redraw_amount ();
    }

    private void redraw_amount () {
        add_button.sensitive = amount != 0;
        if (with_cents) {
            amount_label.label = _("$%.2f").printf (((float)amount)/100);
        } else {
            amount_label.label = _("$%d").printf (amount);
        }
    }

    private Gtk.Button create_number_button (int number) {
        var button = new Gtk.Button.with_label (number.to_string ());
        button.clicked.connect (() => {
            amount = amount * 10 + number;
            if (with_cents) {
                amount = int.min (amount, 100000);
            } else {
                amount = int.min (amount, 1000);
            }

            redraw_amount ();
        });

        return button;
    }
}

