/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.BalancePage : Cafeteria.Page {
    public signal void show_others ();

    private Gtk.Button other_button;
    private TargetWaiter search_grid;
    private BalancePreview amount_grid;
    private Gtk.Stack stack;
    private uint timeout_id = 0U;

    public BalancePage () {
        
    }

    construct {
        margin = 12;

        search_grid = new TargetWaiter ();
        amount_grid = new BalancePreview ();

        stack = new Gtk.Stack ();
        stack.transition_type = Gtk.StackTransitionType.CROSSFADE;
        stack.add (search_grid);
        stack.add (amount_grid);

        add (stack);

        other_button = new Gtk.Button.with_label (_("Other Operations"));
        other_button.get_style_context ().add_class ("back-button");
        other_button.clicked.connect (() => {
            show_others ();
            stack.set_visible_child (search_grid);
        });
        
        show_others.connect (() => {
            lock (timeout_id) {
                if (timeout_id != 0U) {
                    GLib.Source.remove (timeout_id);
                    timeout_id = 0U;
                }
            }
        });
    }

    public override unowned Gtk.Widget get_start_title () {
        return other_button;
    }

    public override unowned Gtk.Widget? get_end_title () {
        return null;
    }

    public override string get_name () {
        return _("Balance");
    }

    public async void card_found (string id) {
        unowned Database db = Database.get_default ();
        var user = yield db.get_user_from_device (id);
        if (user == null) {
            show_others ();
            return;
        }

        amount_grid.set_user_info (user);
        stack.set_visible_child (amount_grid);
        lock (timeout_id) {
            timeout_id = Timeout.add_seconds (5, () => {
                lock (timeout_id) {
                    timeout_id = 0U;
                }

                if (((Gtk.Stack) get_parent ()).get_visible_child () == this) {
                    show_others ();
                }

                return GLib.Source.REMOVE;
            });
        }
    }
}
