/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.CardPage : Cafeteria.Page {
    public signal void show_others ();
    public signal void waiting_target ();
    public signal void stop_waiting_target ();

    private Gtk.Button other_button;
    private Gtk.Grid search_grid;
    private Gtk.Stack stack;
    private Gtk.ListStore list_store;
    private int64 user_id = -1;
    private Gtk.ListBox listbox;
    private Gtk.Grid user_grid;
    private Gtk.SpinButton card_spin;

    public CardPage () {
        
    }

    construct {
        margin = 12;

        var card_promo_label = new Gtk.Label (_("Promo:"));
        card_promo_label.halign = Gtk.Align.START;

        var datetime = new DateTime.now_local ();
        int max_promo = datetime.get_year () + 2;
        if (datetime.get_month () > 8) {
            max_promo++;
        }

        card_spin = new Gtk.SpinButton.with_range (1952, max_promo, 1);
        card_spin.halign = Gtk.Align.START;
        card_spin.value = max_promo;

        list_store = new Gtk.ListStore (2, typeof (string), typeof (int64));
        list_store.set_sort_column_id (0, Gtk.SortType.ASCENDING);

        // The View:
        var view = new Gtk.TreeView.with_model (list_store);
        view.headers_visible = false;
        view.activate_on_single_click = true;

        Gtk.CellRendererText cell = new Gtk.CellRendererText ();
        view.insert_column_with_attributes (-1, null, cell, "markup", 0);
        var scrolled = new Gtk.ScrolledWindow (null, null);
        scrolled.expand = true;
        scrolled.add (view);

        var next_button = new Gtk.Button.with_label (_("Next"));
        next_button.hexpand = true;
        next_button.halign = Gtk.Align.END;
        next_button.get_style_context ().add_class (Gtk.STYLE_CLASS_SUGGESTED_ACTION);
        next_button.sensitive = false;

        var card_grid = new Gtk.Grid ();
        card_grid.column_spacing = 12;
        card_grid.row_spacing = 6;
        card_grid.attach (card_promo_label, 0, 0, 1, 1);
        card_grid.attach (card_spin, 1, 0, 1, 1);
        card_grid.attach (next_button, 2, 0, 1, 1);
        card_grid.attach (scrolled, 0, 1, 3, 1);

        listbox = new Gtk.ListBox ();
        listbox.selection_mode = Gtk.SelectionMode.NONE;
        var listbox_placeholder = new Gtk.Label (_("No Card"));
        listbox_placeholder.show_all ();
        listbox.set_placeholder (listbox_placeholder);
        var list_scrolled = new Gtk.ScrolledWindow (null, null);
        list_scrolled.expand = true;
        list_scrolled.add (listbox);
        var back_button = new Gtk.Button.with_label (_("Back"));
        back_button.get_style_context ().add_class ("back-button");
        back_button.halign = Gtk.Align.START;
        var add_card_button = new Gtk.Button.with_label (_("Add Card"));
        add_card_button.halign = Gtk.Align.END;
        user_grid = new Gtk.Grid ();
        user_grid.column_spacing = 12;
        user_grid.row_spacing = 6;
        user_grid.attach (back_button, 0, 0, 1, 1);
        user_grid.attach (add_card_button, 1, 0, 1, 1);
        user_grid.attach (list_scrolled, 0, 1, 2, 1);

        var target_grid = new TargetWaiter ();
        var search_cancel_button = new Gtk.Button.with_label (_("Cancel"));
        search_cancel_button.get_style_context ().add_class (Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION);
        search_cancel_button.halign = Gtk.Align.CENTER;

        search_grid = new Gtk.Grid ();
        search_grid.orientation = Gtk.Orientation.VERTICAL;
        search_grid.add (target_grid);
        search_grid.add (search_cancel_button);

        stack = new Gtk.Stack ();
        stack.transition_type = Gtk.StackTransitionType.CROSSFADE;
        stack.add (card_grid);
        stack.add (search_grid);
        stack.add (user_grid);

        add (stack);

        other_button = new Gtk.Button.with_label (_("Other Operations"));
        other_button.get_style_context ().add_class ("back-button");
        other_button.clicked.connect (() => {
            show_others ();
            stack.set_visible_child (card_grid);
        });

        next_button.clicked.connect (() => {
            show_user.begin ();
            stack.set_visible_child (user_grid);
        });

        back_button.clicked.connect (() => {
            stack.set_visible_child (card_grid);
        });

        add_card_button.clicked.connect (() => {
            stack.set_visible_child (search_grid);
            waiting_target ();
        });

        search_cancel_button.clicked.connect (() => {
            stack.set_visible_child (user_grid);
            stop_waiting_target ();
        });

        card_spin.value_changed.connect (() => {
            next_button.sensitive = false;
            retrieve_users.begin ((uint) card_spin.value);
        });

        view.row_activated.connect ((path, column) => {
            Gtk.TreeIter iter;
            list_store.get_iter (out iter, path);
            Value user_id_value;
            list_store.get_value (iter, 1, out user_id_value);
            user_id = user_id_value.get_int64 ();
            next_button.sensitive = true;
        });
    }

    public override unowned Gtk.Widget get_start_title () {
        return other_button;
    }

    public override unowned Gtk.Widget? get_end_title () {
        return null;
    }

    public override string get_name () {
        return _("Card Manager");
    }

    private async void show_user () {
        unowned Database db = Database.get_default ();
        var infos = yield db.get_cards (user_id);
        listbox.get_children ().foreach ((child) => child.destroy ());
        foreach (var info in infos) {
            var row = new CardRow (info);
            row.removed.connect (() => {
                db.remove_card.begin (user_id, row.uid);
                row.destroy ();
            });

            row.show_all ();
            listbox.add (row);
        }
    }

    public void check_first_show () {
        if (list_store.iter_n_children (null) == 0) {
            retrieve_users.begin ((uint) card_spin.value);
        }
    }

    public async void retrieve_users (uint promo) {
        unowned Database db = Database.get_default ();
        var infos = yield db.get_users_for_promo (promo);
        list_store.clear ();
        foreach (var info in infos) {
            Gtk.TreeIter iter;
            list_store.append (out iter);
            list_store.set (iter, 0, info.get_pretty_name_markup (), 1, info.id);
        }
    }

    public async void card_found (string id) {
        unowned Database db = Database.get_default ();
        var user = yield db.get_user_from_device (id);
        if (user != null) {
            show_message (_("Card already used by %s.").printf (user.get_pretty_name ()));
            waiting_target ();
            return;
        }

        var type_list_box = new Gtk.ListBox ();
        type_list_box.selection_mode = Gtk.SelectionMode.BROWSE;
        var scrolled = new Gtk.ScrolledWindow (null, null);
        scrolled.expand = true;
        scrolled.add (type_list_box);

        var action_label = new Gtk.Label (_("Card Name:"));
        action_label.halign = Gtk.Align.START;
        action_label.hexpand = true;

        foreach (var code in CardInfo.get_code_names ()) {
            type_list_box.add (new CardTypeRow (code));
        }

        var add_button = new Gtk.Button.with_label (_("Add"));
        add_button.halign = Gtk.Align.END;
        add_button.get_style_context ().add_class (Gtk.STYLE_CLASS_SUGGESTED_ACTION);
        var type_grid = new Gtk.Grid ();
        type_grid.row_spacing = 6;
        type_grid.attach (action_label, 0, 0, 1, 1);
        type_grid.attach (add_button, 1, 0, 1, 1);
        type_grid.attach (scrolled, 0, 1, 2, 1);
        type_grid.show_all ();
        stack.add (type_grid);
        stack.set_visible_child (type_grid);

        add_button.clicked.connect (() => {
            add_button.sensitive = false;
            var type_name = ((CardTypeRow) type_list_box.get_selected_row ()).type_name;
            add_card.begin (id, type_name, (obj, res) => {
                add_card.end (res);
                type_grid.destroy ();
            });
        });
    }

    public async void add_card (string id, string type_name) {
        unowned Database db = Database.get_default ();
        var result = yield db.add_card (user_id, id, type_name);
        if (!result) {
            show_message (_("Impossible to add the card."));
            waiting_target ();
        } else {
            yield show_user ();
            stack.set_visible_child (user_grid);
        }
    }

    public class CardTypeRow : Gtk.ListBoxRow {
        public string type_name { get; construct; }
        public CardTypeRow (string type_name) {
            Object (type_name: type_name);
            show_all ();
        }

        construct {
            var name_label = new Gtk.Label (CardInfo.get_pretty_name (type_name));
            name_label.margin = 6;
            add (name_label);
        }
    }

    public class CardRow : Gtk.ListBoxRow {
        public signal void removed ();
        public string uid { get; construct; }
        private Gtk.Label name_label;
        public CardRow (CardInfo info) {
            Object (uid: info.uid);
            name_label.label = CardInfo.get_pretty_name (info.name);
        }

        construct {
            name_label = new Gtk.Label (null);
            name_label.xalign = 0;
            name_label.hexpand = true;
            var remove_button = new Gtk.Button.from_icon_name ("edit-delete-symbolic", Gtk.IconSize.BUTTON);
            remove_button.get_style_context ().add_class (Gtk.STYLE_CLASS_FLAT);
            remove_button.get_style_context ().add_class (Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION);
            var grid = new Gtk.Grid ();
            grid.orientation = Gtk.Orientation.HORIZONTAL;
            grid.add (name_label);
            grid.add (remove_button);
            grid.margin = 6;
            add (grid);
            remove_button.clicked.connect (() => removed ());
        }
    }
}
