/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.PayPage : Cafeteria.Page {
    public signal void show_cart ();
    public signal void show_home ();

    private Gtk.Button cart_button;
    private Gtk.Grid search_grid;
    private Gtk.Grid amount_grid;
    private Gtk.Stack stack;
    private Gtk.Label name_label;
    private Gtk.Label balance_label;
    private Gtk.Label amount_label;
    private unowned CartView cart_view;

    private uint amount = 0U;
    private uint member_amount = 0U;

    public PayPage (CartView cart_view) {
        this.cart_view = cart_view;
    }

    construct {
        stack = new Gtk.Stack ();
        stack.transition_type = Gtk.StackTransitionType.CROSSFADE;
        margin = 12;
        search_grid = new Gtk.Grid ();
        search_grid.orientation = Gtk.Orientation.VERTICAL;
        search_grid.row_spacing = 12;

        amount_label = new Gtk.Label (null);
        amount_label.halign = Gtk.Align.CENTER;

        var info_label = new Gtk.Label (_("Put the Card over the Target"));
        info_label.get_style_context ().add_class ("h1");
        info_label.expand = true;
        info_label.valign = Gtk.Align.END;
        info_label.wrap = true;
        var spinner = new Gtk.Spinner ();
        spinner.valign = Gtk.Align.START;
        spinner.expand = true;
        spinner.start ();
        var money_button = new Gtk.Button.with_label (_("Pay with cash…"));
        money_button.halign = Gtk.Align.CENTER;
        search_grid.add (amount_label);
        search_grid.add (info_label);
        search_grid.add (spinner);
        search_grid.add (money_button);

        amount_grid = new Gtk.Grid ();
        amount_grid.orientation = Gtk.Orientation.VERTICAL;
        amount_grid.row_spacing = 12;
        name_label = new Gtk.Label (null);
        name_label.get_style_context ().add_class ("h1");
        name_label.expand = true;
        name_label.valign = Gtk.Align.END;
        balance_label = new Gtk.Label (null);
        balance_label.expand = true;
        balance_label.valign = Gtk.Align.START;
        balance_label.get_style_context ().add_class ("h2");
        amount_grid.add (name_label);
        amount_grid.add (balance_label);

        stack.add (search_grid);
        stack.add (amount_grid);
        add (stack);

        money_button.clicked.connect (() => {
            show_home ();
        });

        cart_button = new Gtk.Button.with_label (_("Cart"));
        cart_button.get_style_context ().add_class ("back-button");
        cart_button.clicked.connect (() => show_cart ());
    }

    public override unowned Gtk.Widget get_start_title () {
        return cart_button;
    }

    public override unowned Gtk.Widget? get_end_title () {
        return null;
    }

    public override string get_name () {
        return _("Pay");
    }

    public void update_amount () {
        var price_label = Utils.format_money (cart_view.get_price (false));
        var price_member_label = Utils.format_money (cart_view.get_price (true));
        amount_label.label = _("Due: %s (%s for members)").printf (price_label, price_member_label);
    }

    public async bool card_found (string id) {
        unowned Database db = Database.get_default ();
        var user = yield db.get_user_from_device (id);
        if (user == null) {
            return false;
        }

        uint amount = cart_view.get_price (user.member);
        if (user.money >= amount) {
            yield db.user_pay (user.id, (int)amount);
            cart_view.paid ();
            name_label.label = user.get_pretty_name ();
            balance_label.label = Utils.format_money (user.money - amount);
            stack.set_visible_child (amount_grid);
            Timeout.add_seconds (2, () => {
                show_home ();
                stack.set_visible_child (search_grid);
                return GLib.Source.REMOVE;
            });

            return true;
        } else {
            if (user.money > 0U) {
                show_message (_("Only %s on the account, please add more money.").printf (Utils.format_money (user.money)));
            } else {
                show_message (_("No money on the account, please add some."));
            }

            return false;
        }
    }
}
