/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.ProductsPage : Cafeteria.Page {
    public signal void show_others ();
    public signal void show_pay ();

    private Gtk.Stack stack;
    private ProductListBox drink_list;
    private ProductListBox food_list;
    private Gtk.Button other_button;
    private Gtk.Button pay_button;
    public CartView cart_view;

    public ProductsPage () {
        unowned Database db = Database.get_default ();
        db.new_product.connect ((product) => {
            switch (product.category) {
                case Product.Category.DRINK:
                    drink_list.add_product (product);
                    break;
                case Product.Category.FOOD:
                    food_list.add_product (product);
                    break;
            }
        });
    }

    construct {
        orientation = Gtk.Orientation.HORIZONTAL;

        stack = new Gtk.Stack ();
        stack.transition_type = Gtk.StackTransitionType.SLIDE_LEFT_RIGHT;
        stack.expand = true;
        var switcher = new Gtk.StackSwitcher ();
        switcher.halign = Gtk.Align.CENTER;
        switcher.set_stack (stack);

        var left_grid = new Gtk.Grid ();
        left_grid.orientation = Gtk.Orientation.VERTICAL;
        left_grid.margin_top = 12;
        left_grid.row_spacing = 12;
        left_grid.add (switcher);
        left_grid.add (stack);

        drink_list = new ProductListBox (Product.Category.DRINK);
        var drink_scrolled = new Gtk.ScrolledWindow (null, null);
        drink_scrolled.margin = 12;
        drink_scrolled.margin_top = 0;
        drink_scrolled.add (drink_list);
        stack.add_titled (drink_scrolled, "drink", _("Drinks"));

        food_list = new ProductListBox (Product.Category.FOOD);
        var food_scrolled = new Gtk.ScrolledWindow (null, null);
        food_scrolled.margin = 12;
        food_scrolled.margin_top = 0;
        food_scrolled.add (food_list);
        stack.add_titled (food_scrolled, "food", _("Food"));

        var amount_widget = new AmountWidget ();
        amount_widget.with_cents = true;
        amount_widget.margin = 12;
        amount_widget.margin_top = 0;

        stack.add_titled (amount_widget, "custom", _("Custom"));

        cart_view = new CartView ();

        add (left_grid);
        add (new Gtk.Separator (Gtk.Orientation.VERTICAL));
        add (cart_view);

        other_button = new Gtk.Button.with_label (_("Other Operations"));
        other_button.get_style_context ().add_class ("back-button");

        pay_button = new Gtk.Button.with_label (_("Pay"));
        pay_button.sensitive = false;
        pay_button.get_style_context ().add_class (Gtk.STYLE_CLASS_SUGGESTED_ACTION);

        pay_button.clicked.connect (() => show_pay ());
        other_button.clicked.connect (() => show_others ());
        drink_list.add_to_cart.connect ((prod) => cart_view.add_product (prod));
        food_list.add_to_cart.connect ((prod) => cart_view.add_product (prod));
        amount_widget.accepted.connect ((amount) => {
            var product = new Product ();
            product.name = _("Custom price (%.2f)").printf (((float)amount)/100);
            product.price = amount;
            product.price_member = amount;
            product.price_menu = amount;
            product.price_member_menu = amount;
            cart_view.add_product (product);
        });
        cart_view.product_number.connect ((sens) => {
            pay_button.sensitive = sens;
        });
    }

    public override unowned Gtk.Widget get_start_title () {
        return other_button;
    }

    public override unowned Gtk.Widget? get_end_title () {
        return pay_button;
    }

    public override string get_name () {
        return _("Home");
    }

    public void clear () {
        cart_view.clear ();
    }
}
