/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.RefillPage : Cafeteria.Page {
    public signal void show_home ();
    public signal void show_balance ();
    public signal void waiting_target ();

    private Gtk.Button home_button;
    private Gtk.Stack stack;
    private BalancePreview balance_preview;
    private AmountWidget amount_widget;
    private uint current_amount = 0U;

    public RefillPage () {
        
    }

    construct {
        margin = 12;
        column_spacing = 12;
        row_spacing = 6;

        stack = new Gtk.Stack ();
        stack.transition_type = Gtk.StackTransitionType.CROSSFADE;
        amount_widget = new AmountWidget ();
        var target_waiter = new TargetWaiter ();
        balance_preview = new BalancePreview ();
        add (stack);
        stack.add (amount_widget);
        stack.add (target_waiter);
        stack.add (balance_preview);

        home_button = new Gtk.Button.with_label (_("Home"));
        home_button.get_style_context ().add_class ("back-button");
        home_button.clicked.connect (() => {
            show_home ();
            amount_widget.reset ();
            current_amount = 0;
            stack.set_visible_child (amount_widget);
        });

        amount_widget.accepted.connect ((amount) => {
            current_amount = amount*100;
            stack.set_visible_child (target_waiter);
            waiting_target ();
        });
    }

    public override unowned Gtk.Widget get_start_title () {
        return home_button;
    }

    public override unowned Gtk.Widget? get_end_title () {
        return null;
    }

    public override string get_name () {
        return _("Other Operations");
    }

    public async void card_found (string id) {
        unowned Database db = Database.get_default ();
        var user = yield db.get_user_from_device (id);
        if (user == null) {
            waiting_target ();
            return;
        }

        yield db.user_refill (user.id, (int)current_amount);
        user.money += current_amount;
        balance_preview.set_user_info (user);
        stack.set_visible_child (balance_preview);
    }
}
