/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.StockPage : Cafeteria.Page {
    public signal void show_others ();

    private Gtk.Button other_button;
    private Gtk.ListBox product_list;

    public StockPage () {
        
    }

    construct {
        margin = 12;
        column_spacing = 12;
        row_spacing = 6;
        orientation = Gtk.Orientation.VERTICAL;

        var save_button = new Gtk.Button.with_label (_("Save changes"));
        save_button.get_style_context ().add_class (Gtk.STYLE_CLASS_SUGGESTED_ACTION);
        save_button.halign = Gtk.Align.CENTER;
        save_button.clicked.connect (() => {
            save_products ();
        });

        other_button = new Gtk.Button.with_label (_("Other Operations"));
        other_button.get_style_context ().add_class ("back-button");
        other_button.clicked.connect (() => {
            show_others ();
        });

        product_list = new Gtk.ListBox ();
        product_list.selection_mode = Gtk.SelectionMode.NONE;
        var list_scrolled = new Gtk.ScrolledWindow (null, null);
        list_scrolled.expand = true;
        list_scrolled.add (product_list);
        add (list_scrolled);
        add (save_button);
    }

    public override unowned Gtk.Widget get_start_title () {
        return other_button;
    }

    public override unowned Gtk.Widget? get_end_title () {
        return null;
    }

    public override string get_name () {
        return _("Stock Management");
    }

    public void show_products () {
        unowned Database db = Database.get_default ();
        product_list.get_children ().foreach ((child) => {
            child.destroy ();
        });

        foreach (var product in db.get_products ()) {
            if (product.children.is_empty) {
                var row = new StockRow (product);
                row.show_all ();
                product_list.add (row);
            }
        }
    }

    private void save_products () {
        product_list.get_children ().foreach ((child) => {
            var row = (StockRow) child;
            var stock = row.get_stock ();
            var product = row.product;
            if (stock != product.quantity) {
                save_product_stock.begin (product, stock);
            }
        });
    }

    private async void save_product_stock (Product prod, int stock) {
        unowned Database db = Database.get_default ();
        var result = yield db.save_stock (prod, stock);
    }

    public class StockRow : Gtk.ListBoxRow {
        public Product product { get; construct; }
        private Gtk.SpinButton stock_spin;
        public StockRow (Product product) {
            Object (product: product);
        }

        construct {
            var grid = new Gtk.Grid ();
            grid.margin = 6;
            grid.orientation = Gtk.Orientation.HORIZONTAL;
            var product_label = new Gtk.Label (product.name);
            product_label.hexpand = true;
            product_label.xalign = 0;
            stock_spin = new Gtk.SpinButton.with_range (-1, int.MAX, 1);
            stock_spin.value = product.quantity;
            grid.add (product_label);
            grid.add (stock_spin);
            add (grid);
        }

        public int get_stock () {
            return (int)stock_spin.value;
        }
    }
}
