/* Copyright 2015 Corentin Noël <corentin@elementary.io>
*
* This file is part of Cafeteria
*
* Cafeteria is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* Cafeteria is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with Cafeteria. If not, see http://www.gnu.org/licenses/.
*/

public class Cafeteria.ProductListBox : Gtk.ListBox {
    public signal void add_to_cart (Product product);

    public Product.Category category { get; construct; }

    public ProductListBox (Product.Category category) {
        Object (category: category);
    }

    construct {
        activate_on_single_click = true;
        set_filter_func (filter_func);
        set_sort_func (sort_func);
        set_header_func (header_func);
    }

    private static bool filter_func (Gtk.ListBoxRow child) {
        return ((ProductRow) child).product.quantity != 0;
    }

    private int sort_func (Gtk.ListBoxRow row1, Gtk.ListBoxRow row2) {
        var product1 = ((ProductRow) row1).product;
        var product2 = ((ProductRow) row2).product;
        var parent1 = product1.parent;
        var parent2 = product2.parent;
        if (parent1 == parent2) {
            return product1.name.collate (product2.name);
        } else {
            return parent1.name.collate (parent2.name);
        }
    }

    private static void header_func (Gtk.ListBoxRow row, Gtk.ListBoxRow? before) {
        var parent = ((ProductRow) row).product.parent;
        if (before == null || parent == null || parent != ((ProductRow) before).product.parent) {
            var parent_label = new Gtk.Label (null);
            parent_label.get_style_context ().add_class ("h3");
            parent_label.halign = Gtk.Align.START;
            parent_label.margin = 3;
            if (parent == null) {
                parent_label.label = ((ProductRow) row).product.name;
            } else {
                parent_label.label = parent.name;
            }
            row.set_header (parent_label);
        } else {
            row.set_header (null);
        }
    }

    public void add_product (Product product) {
        foreach (var subprod in product.children) {
            var product_row = new ProductRow (subprod);
            product_row.show_all ();
            add (product_row);
        }
    }

    public override void row_activated (Gtk.ListBoxRow row) {
        var product = ((ProductRow) row).product;
        add_to_cart (product);
    }

    public class ProductRow : Gtk.ListBoxRow {
        public Product product { get; construct; }
        public ProductRow (Product prod) {
            Object (product: prod);
        }

        construct {
            var label = new Gtk.Label (null);
            label.halign = Gtk.Align.START;
            label.margin = 6;
            label.margin_start = 12;
            add (label);

            product.notify["name"].connect (changed);
            product.notify["quantity"].connect (changed);
            product.bind_property ("name", label, "label", BindingFlags.SYNC_CREATE);
        }
    }
}
