
public class Cafeteria.Reader : GLib.Object {
    public signal void error_message (string msg);
    public signal void tag_found (string UID);
    private static const uint32 SPEED = 5000;
    private static const uint8 GPIO = 18;
    private GLib.Cancellable cancellable;

    public Reader () {
        cancellable = new GLib.Cancellable ();
        if (HW_init (SPEED, GPIO)) {
            Posix.setgid (1002);
            Posix.setuid (500);
            RC522.init ();
        }
    }

    ~Reader () {
            BCM2835.SPI.end();
            BCM2835.close();
    }

    public void start_poll () {
        cancellable.reset ();
        new GLib.Thread<void*> ("nfc", () => {
            while (!cancellable.is_cancelled ()) {
                uint16 card_type;
                var status = RFID.find_tag (out card_type);
                if (status == Tag.NOTAG) {
                    Posix.usleep (200000);
                    continue;
                } else if ((status != Tag.OK) && (status != Tag.COLLISION)) {
                    continue;
                }

                string sn;
                if (RFID.select_tag_sn (out sn) != Tag.OK) {
                    continue;
                } else {
                    sn = "%04x%s".printf (card_type, sn).up ();
                    Idle.add (() => {
                        tag_found (sn);
                        return Source.REMOVE;
                    });

                    cancellable.cancel ();
                }
            }

            return null;
        });
    }

    public void stop_poll () {
        cancellable.cancel ();
    }

    private bool HW_init (uint32 spi_speed, uint8 gpio) {
        uint16 sp = (uint16)(250000L/spi_speed);
        if (!BCM2835.init ()) {
            error_message ("Impossible d'initialiser la carte NFC (BCM2835) !");
            return false;
        }

        if (gpio < 28) {
            BCM2835.GPIO.fsel (gpio, BCM2835.GPIO.FunctionSelect.OUTP);
            BCM2835.GPIO.write (gpio, BCM2835.Bit.LOW);
        }

        BCM2835.SPI.begin ();
        BCM2835.SPI.set_bit_order (BCM2835.SPI.BitOrder.MSBFIRST);      // The default
        BCM2835.SPI.set_data_mode (BCM2835.SPI.Mode.MODE0);                   // The default
        BCM2835.SPI.set_clock_divider ((BCM2835.SPI.ClockDivider)sp); // The default
        BCM2835.SPI.chip_select (BCM2835.SPI.ChipSelect.CS0);                      // The default
        BCM2835.SPI.set_chip_select_polarity (BCM2835.SPI.ChipSelect.CS0, BCM2835.Bit.LOW);      // the default
        return true;
    }
}
