[CCode (cprefix = "bcm2835_", lower_case_cprefix = "bcm2835_", cheader_filename = "bcm2835.h")]
namespace BCM2835 {
	public enum Bit {
		[CCode (cname = "HIGH")]
		HIGH,
		[CCode (cname = "LOW")]
		LOW;
	}

	public static bool init ();
	public static bool close ();
	namespace GPIO {
		[CCode (cname = "bcm2835FunctionSelect", cprefix = "BCM2835_GPIO_FSEL_", has_type_id = false)]
		public enum FunctionSelect {
			INPT,   ///< Input
			OUTP,   ///< Output
			ALT0,   ///< Alternate function 0
			ALT1,   ///< Alternate function 1
			ALT2,   ///< Alternate function 2
			ALT3,   ///< Alternate function 3
			ALT4,   ///< Alternate function 4
			ALT5,   ///< Alternate function 5
			MASK;   ///< Function select bits mask
		}

		// RPi Version 1
		[CCode (cname = "RPiGPIOPin", cprefix = "RPI_GPIO_P1", has_type_id = false)]
		public enum PinRP1 {
			_03,  ///< Version 1, Pin P1-03
			_05,  ///< Version 1, Pin P1-05
			_07,  ///< Version 1, Pin P1-07
			_08,  ///< Version 1, Pin P1-08, defaults to alt function 0 UART0_TXD
			_10,  ///< Version 1, Pin P1-10, defaults to alt function 0 UART0_RXD
			_11,  ///< Version 1, Pin P1-11
			_12,  ///< Version 1, Pin P1-12
			_13,  ///< Version 1, Pin P1-13
			_15,  ///< Version 1, Pin P1-15
			_16,  ///< Version 1, Pin P1-16
			_18,  ///< Version 1, Pin P1-18
			_19,  ///< Version 1, Pin P1-19, MOSI when SPI0 in use
			_21,  ///< Version 1, Pin P1-21, MISO when SPI0 in use
			_22,  ///< Version 1, Pin P1-22
			_23,  ///< Version 1, Pin P1-23, CLK when SPI0 in use
			_24,  ///< Version 1, Pin P1-24, CE0 when SPI0 in use
			_26;  ///< Version 1, Pin P1-26, CE1 when SPI0 in use
		}

		// RPi Version 2
		[CCode (cname = "RPiGPIOPin", cprefix = "RPI_V2_GPIO_P1", has_type_id = false)]
		public enum PinRP2 {
			_03,  ///< Version 2, Pin P1-03
			_05,  ///< Version 2, Pin P1-05
			_07,  ///< Version 2, Pin P1-07
			_08,  ///< Version 2, Pin P1-08, defaults to alt function 0 UART0_TXD
			_10,  ///< Version 2, Pin P1-10, defaults to alt function 0 UART0_RXD
			_11,  ///< Version 2, Pin P1-11
			_12,  ///< Version 2, Pin P1-12
			_13,  ///< Version 2, Pin P1-13
			_15,  ///< Version 2, Pin P1-15
			_16,  ///< Version 2, Pin P1-16
			_18,  ///< Version 2, Pin P1-18
			_19,  ///< Version 2, Pin P1-19, MOSI when SPI0 in use
			_21,  ///< Version 2, Pin P1-21, MISO when SPI0 in use
			_22,  ///< Version 2, Pin P1-22
			_23,  ///< Version 2, Pin P1-23, CLK when SPI0 in use
			_24,  ///< Version 2, Pin P1-24, CE0 when SPI0 in use
			_26;  ///< Version 2, Pin P1-26, CE1 when SPI0 in use
		}

		// RPi Version 2, new plug P5
		[CCode (cname = "RPiGPIOPin", cprefix = "RPI_V2_GPIO_P5", has_type_id = false)]
		public enum PinRP2_P5 {
			_03,  ///< Version 2, Pin P5-03
			_04,  ///< Version 2, Pin P5-04
			_05,  ///< Version 2, Pin P5-05
			_06;  ///< Version 2, Pin P5-06
		}

		public static void fsel (uint8 gpio, FunctionSelect function);
		public static void write (uint8 gpio, Bit bit);
	}
	namespace SPI {
		[CCode (cname = "bcm2835SPIBitOrder", cprefix = "BCM2835_SPI_BIT_ORDER_", has_type_id = false)]
		public enum BitOrder {
			LSBFIRST,  ///< LSB First
			MSBFIRST;  ///< MSB First
		}

		[CCode (cname = "bcm2835SPIMode", cprefix = "BCM2835_SPI_", has_type_id = false)]
		public enum Mode {
			MODE0,  ///< CPOL = 0, CPHA = 0
			MODE1,  ///< CPOL = 0, CPHA = 1
			MODE2,  ///< CPOL = 1, CPHA = 0
			MODE3;  ///< CPOL = 1, CPHA = 1
		}

		[CCode (cname = "bcm2835SPIChipSelect", cprefix = "BCM2835_SPI_", has_type_id = false)]
		public enum ChipSelect {
			CS0,  ///< CPOL = 0, CPHA = 0
			CS1,  ///< CPOL = 0, CPHA = 1
			CS2,  ///< CPOL = 1, CPHA = 0
			CS_NONE;  ///< CPOL = 1, CPHA = 1
		}

		[CCode (cname = "bcm2835SPIClockDivider", cprefix = "BCM2835_SPI_", has_type_id = false)]
		public enum ClockDivider {
			CLOCK_DIVIDER_65536,   ///< 65536 = 262.144us = 3.814697260kHz
			CLOCK_DIVIDER_32768,   ///< 32768 = 131.072us = 7.629394531kHz
			CLOCK_DIVIDER_16384,   ///< 16384 = 65.536us = 15.25878906kHz
			CLOCK_DIVIDER_8192,    ///< 8192 = 32.768us = 30/51757813kHz
			CLOCK_DIVIDER_4096,    ///< 4096 = 16.384us = 61.03515625kHz
			CLOCK_DIVIDER_2048,    ///< 2048 = 8.192us = 122.0703125kHz
			CLOCK_DIVIDER_1024,    ///< 1024 = 4.096us = 244.140625kHz
			CLOCK_DIVIDER_512,     ///< 512 = 2.048us = 488.28125kHz
			CLOCK_DIVIDER_256,     ///< 256 = 1.024us = 976.5625MHz
			CLOCK_DIVIDER_128,     ///< 128 = 512ns = = 1.953125MHz
			CLOCK_DIVIDER_64,      ///< 64 = 256ns = 3.90625MHz
			CLOCK_DIVIDER_32,      ///< 32 = 128ns = 7.8125MHz
			CLOCK_DIVIDER_16,      ///< 16 = 64ns = 15.625MHz
			CLOCK_DIVIDER_8,       ///< 8 = 32ns = 31.25MHz
			CLOCK_DIVIDER_4,       ///< 4 = 16ns = 62.5MHz
			CLOCK_DIVIDER_2,       ///< 2 = 8ns = 125MHz, fastest you can get
			CLOCK_DIVIDER_1;       ///< 0 = 262.144us = 3.814697260kHz, same as 0/65536
		}

		public static void begin ();
		public static void end ();
		public static void spi_end ();
		[CCode (cname = "bcm2835_spi_setBitOrder")]
		public static void set_bit_order (BitOrder bit_order);      // The default
		[CCode (cname = "bcm2835_spi_chipSelect")]
		public static void set_data_mode (Mode mode);                   // The default
		[CCode (cname = "bcm2835_spi_setClockDivider")]
		public static void set_clock_divider (ClockDivider clock_divider); // The default
		[CCode (cname = "bcm2835_spi_chipSelect")]
		public static void chip_select (ChipSelect chip_select);                      // The default
		[CCode (cname = "bcm2835_spi_setChipSelectPolarity")]
		extern void set_chip_select_polarity (ChipSelect chip_select, uint8 active);
		public static uint8 transfer (uint8 val);
		public static void transfern (ref char[] buf);
	}
}

