
[CCode (cheader_filename = "neardal.h", cprefix = "neardal_", lower_case_cprefix = "neardal_")]
namespace NeardAL {
	[Compact]
	[CCode (free_function = "neardal_free_adapter", cname = "neardal_adapter", has_type_id = false)]
	public class Adapter {
		public string name;
		public string mode;
		[CCode (cname = "polling")]
		private short Polling;
		public bool polling {
			get {
				return Polling == 1;
			}
			set {
				Polling = value ? 1 : 0;
			}
		}
		[CCode (cname = "powered")]
		private short Powered;
		public bool powered {
			get {
				return Powered == 1;
			}
			set {
				Powered = value ? 1 : 0;
			}
		}
		[CCode (array_length_cname = "nbProtocols")]
		public string[] protocols;
		[CCode (array_length_cname = "nbTags")]
		public string[] tags;
		[CCode (array_length_cname = "nbDevs")]
		public string[] devs;
	}
	[Compact]
	[CCode (free_function = "neardal_free_tag", cname = "neardal_tag", has_type_id = false)]
	public class Tag {
		public string name;
		[CCode (array_length_cname = "nbRecords")]
		public string[] records;
		[CCode (array_length_cname = "nbTagTypes")]
		public string[] tagType;
		public unowned string type;
		private short readOnly;
		public bool read_only {
			get {
				return readOnly == 1;
			}
			set {
				readOnly = value ? 1 : 0;
			}
		}
		public GLib.Bytes iso14443aAtqa;
		public GLib.Bytes iso14443aSak;
		public GLib.Bytes iso14443aUid;
		public GLib.Bytes felicaManufacturer;
		public GLib.Bytes felicaCid;
		public GLib.Bytes felicaIc;
		public GLib.Bytes felicaMaxRespTimes;
	}
	[Compact]
	[CCode (free_function = "neardal_free_device", cname = "neardal_dev", has_type_id = false)]
	public class Device {
		public string name;
		[CCode (array_length_cname = "nbRecords")]
		public string[] records;
	}
	[Compact]
	[CCode (free_function = "neardal_free_record", cname = "neardal_record", has_type_id = false)]
	public class Record {
		public string action;
		public string carrier;
		public string encoding;
		public string language;
		public string mime;
		public string name;
		public string representation;
		public string type;

		public string ssid;
		public string passphrase;
		public string encryption;
		public string authentication;

		public string uri;

		public uint uriObjSize;

		[CCode (cname = "neardal_dev_push")]
		public ErrorCode device_push ();
		[CCode (cname = "neardal_tag_write")]
		public ErrorCode tag_write ();
		[CCode (cname = "neardal_record_to_g_variant")]
		public GLib.Variant to_variant ();
		[CCode (cname = "neardal_g_variant_to_record")]
		static Record from_variant (GLib.Variant variant);
	}
	[CCode (cprefix = "NEARD_ADP_MODE_", has_type_id = false)]
	public enum AdapterMode {
		INITIATOR,
		TARGET,
		DUAL
	}
	[CCode (cprefix = "NEARDAL_ERROR_", cname = "errorCode_t")]
	public enum ErrorCode {
		[CCode (cname = "NEARDAL_SUCCESS")]
		SUCCESS,
		GENERAL_ERROR,
		INVALID_PARAMETER,
		NO_MEMORY,
		DBUS,
		DBUS_CANNOT_CREATE_PROXY,
		DBUS_CANNOT_INVOKE_METHOD,
		NO_ADAPTER,
		POLLING_ALREADY_ACTIVE,
		NO_TAG,
		NO_RECORD,
		INVALID_RECORD,
		[CCode (cname = "NEARDAL_ERROR_NO_DEV")]
		NO_DEVICE,
		DBUS_INVOKE_METHOD_ERROR;
		[CCode (cname = "neardal_error_get_text")]
		public unowned string get_text ();
	}

	public ErrorCode start_poll_loop (string adapter_name, AdapterMode mode);
	public ErrorCode start_poll (string adapter_name);
	public ErrorCode stop_poll (string adapter_name);

	public ErrorCode get_adapters (out string[] array);
	public ErrorCode get_adapter_properties (string adpName, out Adapter adapter);
	public ErrorCode set_adapter_properties (string adpName, int adapter_property_id, void* value);
	[CCode (cname = "set_cb_adapter_property_changed")]
	public ErrorCode set_adapter_property_changed_callback (AdapterPropertyCallback property_changed_callback);
	[CCode (cname = "neardal_set_cb_adapter_added")]
	public ErrorCode set_adapter_added_callback (AdapterCallback adapter_added_callback);
	[CCode (cname = "neardal_set_cb_adapter_removed")]
	public ErrorCode set_adapter_removed_callback (AdapterCallback adapter_removed_callback);

	public ErrorCode get_devices (string adapter_name, out string[] array);
	[CCode (cname = "neardal_get_dev_properties")]
	public ErrorCode get_device_properties (string device_name, out Device device);
	[CCode (cname = "neardal_set_cb_dev_found")]
	public ErrorCode set_dev_found_callback (DeviceCallback device_found_callback);
	[CCode (cname = "neardal_set_cb_dev_lost")]
	public ErrorCode set_dev_lost_callback (DeviceCallback device_lost_callback);

	public ErrorCode get_tags (string adapter_name, out string[] array);
	public ErrorCode get_tag_properties (string tag_name, out Tag tag);
	[CCode (cname = "neardal_set_cb_tag_found")]
	public ErrorCode set_tag_found_callback (TagCallback tag_found_callback);
	[CCode (cname = "neardal_set_cb_tag_lost")]
	public ErrorCode set_tag_lost_callback (TagCallback tag_found_callback);

	public ErrorCode get_records (string tag_name, out string[] array);
	public ErrorCode get_record_properties (string record_name, out Record record);
	[CCode (cname = "neardal_set_cb_record_found")]
	public ErrorCode set_record_found_callback (RecordCallback record_found_callback);

	public static void destroy ();

	[CCode (has_type_id = false)]
	public delegate void AdapterCallback (string adapter_name);
	[CCode (has_type_id = false)]
	public delegate void AdapterPropertyCallback (string adapter_name, string property_name, void* value);
	[CCode (has_type_id = false)]
	public delegate void TagCallback (string tag_name);
	[CCode (has_type_id = false)]
	public delegate void DeviceCallback (string adapter_name);
	[CCode (has_type_id = false)]
	public delegate void RecordCallback (string record_name);
	[CCode (has_type_id = false)]
	public delegate void NDEFAgentCallback (uchar[] record_array, uchar[] ndef_array);
}
