[CCode (cprefix = "nfc_", lower_case_cprefix = "nfc_", cheader_filename = "nfc/nfc.h")]
namespace Nfc {
	[Compact]
	[CCode (cheader_filename = "nfc/nfc.h", cname = "nfc_context", free_function = "nfc_exit")]
	public class Context {
		[CCode (has_construct_function = false)]
		private Context ();
		[CCode (cname = "nfc_open")]
		public Nfc.Device open (string? connstring = null);
		[CCode (cname = "nfc_list_devices")]
		public size_t list_devices ([CCode (array_length_pos = 1.1)] string[] connstrings);
	}

	[CCode (cheader_filename = "nfc/nfc.h", cname = "nfc_device", free_function = "nfc_close")]
	[Compact]
	public class Device {
		[CCode (has_construct_function = false)]
		private Device ();
		[CCode (cname = "nfc_device_get_information_about")]
		int get_information (out string informations);
		[CCode (cname = "nfc_abort_command")]
		public int abort_command ();
		[CCode (cname = "nfc_idle")]
		public int idle ();
		[CCode (cname = "nfc_initiator_init")]
		public int initiator_init ();
		[CCode (cname = "nfc_initiator_init_secure_element")]
		public int initiator_init_secure_element ();
		[CCode (cname = "nfc_initiator_select_passive_target")]
		public int initiator_select_passive_target (Nfc.Modulation nm, [CCode (array_length = false, array_null_terminated = false)] uint8[]? pbtInitData, size_t szInitData, out Nfc.Target pnt);
		[CCode (cname = "nfc_initiator_list_passive_targets")]
		public int initiator_list_passive_targets (Nfc.Modulation nm, [CCode (array_length_type = "size_t")] out Nfc.Target ant[]);
		[CCode (cname = "nfc_initiator_poll_target")]
		public int initiator_poll_target ([CCode (array_length_type = "size_t")] Nfc.Modulation[] pnmTargetTypes, uint8 uiPollNr, uint8 uiPeriod, out Nfc.Target pnt);
		[CCode (cname = "nfc_initiator_select_dep_target")]
		public int initiator_select_dep_target (Nfc.DepMode ndm, Nfc.BaudRate nbr, Nfc.DepInfo pndiInitiator, out Nfc.Target pnt, int timeout);
		[CCode (cname = "nfc_initiator_poll_dep_target")]
		public int initiator_poll_dep_target (Nfc.DepMode ndm, Nfc.BaudRate nbr, Nfc.DepInfo pndiInitiator, out Nfc.Target pnt, int timeout);
		[CCode (cname = "nfc_initiator_deselect_target")]
		public int initiator_deselect_target ();
		[CCode (cname = "nfc_initiator_transceive_bytes")]
		public int initiator_transceive_bytes (
				[CCode (array_length_type = "size_t")] uint8[] pbtTx,
				[CCode (array_length_type = "size_t")] out uint8[]? pbtRx,
				int timeout = -1
			);
		[CCode (cname = "nfc_initiator_transceive_bits")]
		public int initiator_transceive_bits (
				[CCode (array_length = false, array_null_terminated = false)] uint8[] pbtTx,
				size_t szTxBits,
				[CCode (array_length = false, array_null_terminated = false)] uint8[]? pbtTxPar,
				[CCode (array_length = false, array_null_terminated = false)] out uint8[]? pbtRx = null,
				size_t? szRx = null,
				[CCode (array_length = false, array_null_terminated = false)] out uint8[]? pbtRxPar = null
			);
		[CCode (cname = "nfc_initiator_transceive_bytes_timed")]
		public int initiator_transceive_bytes_timed (
				[CCode (array_length_type = "size_t")] uint8[] pbtTx,
				[CCode (array_length_type = "size_t")] out uint8[]? pbtRx = null,
				uint32 cycles = 0
			);
		[CCode (cname = "nfc_initiator_transceive_bits_timed")]
		public int initiator_transceive_bits_timed (
				[CCode (array_length = false, array_null_terminated = false)] uint8[] pbtTx,
				size_t szTxBits,
				[CCode (array_length = false, array_null_terminated = false)] uint8[] pbtTxPar,
				[CCode (array_length = false, array_null_terminated = false)] out uint8[]? pbtRx = null,
				size_t? szRx = null,
				[CCode (array_length = false, array_null_terminated = false)] out uint8[]? pbtRxPar = null,
				uint32 cycles = 0
			);
		[CCode (cname = "nfc_initiator_target_is_present")]
		public int initiator_target_is_present (Nfc.Target target);
		[CCode (cname = "nfc_device_set_property_int")]
		public int set_property_int (Nfc.Property property, int value);
		[CCode (cname = "nfc_device_set_property_bool")]
		public int set_property_bool (Nfc.Property property, bool enable);
		[CCode (cname = "nfc_device_get_name")]
		public unowned string get_name ();
		[CCode (cname = "nfc_device_get_name")]
		public unowned string get_connstring ();
		[CCode (cname = "nfc_device_get_supported_modulation")]
		public int get_supported_modulation (Nfc.Mode mode, [CCode (array_length = false, array_null_terminated = false)] out Nfc.ModulationType[] supported_mt);
		[CCode (cname = "nfc_device_get_supported_baud_rate")]
		public int get_supported_baud_rate (Nfc.ModulationType nmt, [CCode (array_length = false, array_null_terminated = false)] out Nfc.BaudRate[] supported_br);
		/* Error reporting */
		[CCode (cname = "nfc_strerror")]
		public string get_error_string ();
		[CCode (cname = "nfc_strerror_r")]
		public Nfc.Error get_error ([CCode (array_length_pos = 1.1)] string error);
		[CCode (cname = "nfc_perror")]
		public void perror (string s);
		[CCode (cname = "nfc_device_get_last_error")]
		public int get_last_error ();
	}

	[Compact]
	public class Driver {
		[CCode (has_construct_function = false)]
		private Driver ();
	}

	[CCode (cname = "nfc_dep_info", has_type_id = false)]
	[SimpleType]
	public struct DepInfo {
		uint8 abtNFCID3[10];
		uint8 btDID;
		uint8 btBS;
		uint8 btBR;
		uint8 btTO;
		uint8 btPP;
		uint8 abtGB[48];
		size_t szGB;
		Nfc.DepMode ndm;
	}

	[CCode (cname = "nfc_felica_info", has_type_id = false)]
	[SimpleType]
	public struct FelicaInfo {
		size_t szLen;
		uint8 btResCode;
		uint8 abtId[8];
		uint8 abtPad[8];
		uint8 abtSysCode[2];
	}

	[CCode (cname = "nfc_iso14443a_info", has_type_id = false)]
	[SimpleType]
	public struct Iso14443aInfo {
		uint8 abtAtqa[2];
		uint8 btSak;
		size_t szUidLen;
		uint8 abtUid[10];
		size_t szAtsLen;
		uint8 abtAts[254];
	}

	[CCode (cname = "nfc_iso14443b_info", has_type_id = false)]
	[SimpleType]
	public struct Iso14443bInfo {
		uint8 abtPupi[4];
		uint8 abtApplicationData[4];
		uint8 abtProtocolInfo[3];
		uint8 ui8CardIdentifier;
	}

	[CCode (cname = "nfc_iso14443bi_info", has_type_id = false)]
	[SimpleType]
	public struct Iso14443biInfo {
		uint8 abtDIV[4];
		uint8 btVerLog;
		uint8 btConfig;
		size_t szAtrLen;
		uint8 abtAtr[33];
	}

	[CCode (cname = "nfc_iso14443b2sr_info", has_type_id = false)]
	[SimpleType]
	public struct Iso14443b2srInfo {
		uint8 abtUID[8];
	}

	[CCode (cname = "nfc_iso14443b2ct_info", has_type_id = false)]
	[SimpleType]
	public struct Iso14443b2ctInfo {
		uint8 abtUID[4];
		uint8 btProdCode;
		uint8 btFabCode;
	}

	[CCode (cname = "nfc_jewel_info", has_type_id = false)]
	[SimpleType]
	public struct JewelInfo {
		uint8 btSensRes[2];
		uint8 btId[4];
	}

	[CCode (cname = "nfc_modulation", has_type_id = false)]
	[SimpleType]
	public struct Modulation {
		Nfc.ModulationType nmt;
		Nfc.BaudRate nbr;
	}

	[CCode (cname = "nfc_target", has_type_id = false)]
	[SimpleType]
	public struct Target {
		[CCode (has_construct_function = false)]
		public Target ();
		Nfc.Modulation nm;
		[CCode (cname = "nti.nai")]
		Nfc.Iso14443aInfo nai;
		[CCode (cname = "nti.nfi")]
		Nfc.FelicaInfo nfi;
		[CCode (cname = "nti.nbi")]
		Nfc.Iso14443bInfo nbi;
		[CCode (cname = "nti.nii")]
		Nfc.Iso14443biInfo nii;
		[CCode (cname = "nti.nsi")]
		Nfc.Iso14443b2srInfo nsi;
		[CCode (cname = "nti.nci")]
		Nfc.Iso14443b2ctInfo nci;
		[CCode (cname = "nti.nji")]
		Nfc.JewelInfo nji;
		[CCode (cname = "nti.ndi")]
		Nfc.DepInfo ndi;
		[CCode (cname = "str_nfc_target", instance_pos = 1.5)]
		public int to_string (out unowned string buf, bool verbose);
	}

	[CCode (cname = "nfc_baud_rate", cprefix = "NBR_", has_type_id = false)]
	public enum BaudRate {
		UNDEFINED,
		[CCode (cname = "NBR_106")]
		_106,
		[CCode (cname = "NBR_212")]
		_212,
		[CCode (cname = "NBR_424")]
		_424,
		[CCode (cname = "NBR_847")]
		_847;
		[CCode (cname = "str_nfc_baud_rate")]
		public unowned string to_string (Nfc.BaudRate nmt);
	}

	[CCode (cname = "nfc_modulation_type", cprefix = "NMT_", has_type_id = false)]
	public enum ModulationType {
		ISO14443A = 1,
		JEWEL,
		ISO14443B,
		ISO14443BI,
		ISO14443B2SR,
		ISO14443B2CT,
		FELICA,
		DEP;
		[CCode (cname = "str_nfc_modulation_type")]
		public unowned string to_string (Nfc.ModulationType nmt);
	}

	[CCode (cname = "nfc_mode", cprefix = "N_", has_type_id = false)]
	public enum Mode {
		TARGET,
		INITIATOR
	}

	[CCode (cname = "nfc_property", cprefix = "NP_", has_type_id = false)]
	public enum Property {
		TIMEOUT_COMMAND,
		TIMEOUT_ATR,
		TIMEOUT_COM,
		HANDLE_CRC,
		HANDLE_PARITY,
		ACTIVATE_FIELD,
		ACTIVATE_CRYPTO1,
		INFINITE_SELECT,
		ACCEPT_INVALID_FRAMES,
		ACCEPT_MULTIPLE_FRAMES,
		AUTO_ISO14443_4,
		EASY_FRAMING,
		FORCE_ISO14443_A,
		FORCE_ISO14443_B,
		FORCE_SPEED_106,
	}

	[CCode (cname = "nfc_dep_mode", cprefix = "NDM_", has_type_id = false)]
	public enum DepMode {
		UNDEFINED,
		PASSIVE,
		ACTIVE
	}

	[CCode (cname = "int", cprefix = "NFC_E", has_type_id = false)]
	public enum Error {
		[CCode (cname = "NFC_SUCCESS")]
		SUCCESS,
		IO,
		INVARG,
		DEVNOTSUPP,
		NOTSUCHDEV,
		OVFLOW,
		TIMEOUT,
		OPABORTED,
		NOTIMPL,
		TGRELEASED,
		RFTRANS,
		MFCAUTHFAIL,
		SOFT,
		CHIP
	}

	/* Library initialization/deinitialization */
	public void init (out Nfc.Context context);
	public int register_driver (Nfc.Driver driver);
	public unowned string version ();
	public void free (void* p);

/* Misc. functions */
	public void iso14443a_crc ([CCode (array_length_pos = 1.1)]uint8[] pbtData, uint8 pbtCrc);
	public void iso14443a_crc_append ([CCode (array_length_pos = 1.1)]uint8[] pbtData);
	public void iso14443b_crc ([CCode (array_length_pos = 1.1)]uint8[] pbtData, uint8 pbtCrc);
	public void iso14443b_crc_append ([CCode (array_length_pos = 1.1)]uint8[] pbtData);
	public uint8 iso14443a_locate_historical_bytes ([CCode (array_length_pos = 1.1)]uint8 pbtAts, size_t pszTk);

	[CCode (cname = "NFC_BUFSIZE_CONNSTRING")]
	public const int BUFSIZE_CONNSTRING;
}

